@@ -4,13 +4,10 @@
 Module where grappelli dashboard modules classes are defined.
 """
 
-# DJANGO IMPORTS
-from django.utils.text import capfirst
-from django.contrib.contenttypes.models import ContentType
-from django.utils.translation import ugettext_lazy as _
 from django.apps import apps as django_apps
+from django.utils.text import capfirst
+from django.utils.translation import gettext_lazy as _
 
-# GRAPPELLI IMPORTS
 from grappelli.dashboard.utils import AppListElementMixin
 
 
@@ -198,6 +195,12 @@ class LinkList(DashboardModule):
                     link_dict['external'] = link[2]
                 if len(link) >= 4:
                     link_dict['description'] = link[3]
+                if len(link) >= 5:
+                    target = link[4]
+                    if isinstance(target, bool):
+                        target = '_blank' if target else None
+                    if target:
+                        link_dict['target'] = str(target)
                 new_children.append(link_dict)
             else:
                 new_children.append(link)
@@ -236,7 +239,8 @@ class AppList(DashboardModule, AppListElementMixin):
                 }
             model_dict = {}
             model_dict['title'] = capfirst(model._meta.verbose_name_plural)
-            if perms['change']:
+            if perms['change'] or perms['view']:
+                model_dict['view_only'] = not perms['change']
                 model_dict['admin_url'] = self._get_admin_change_url(model, context)
             if perms['add']:
                 model_dict['add_url'] = self._get_admin_add_url(model, context)
@@ -274,7 +278,8 @@ class ModelList(DashboardModule, AppListElementMixin):
         for model, perms in items:
             model_dict = {}
             model_dict['title'] = capfirst(model._meta.verbose_name_plural)
-            if perms['change']:
+            if perms['change'] or perms['view']:
+                model_dict['view_only'] = not perms['change']
                 model_dict['admin_url'] = self._get_admin_change_url(model, context)
             if perms['add']:
                 model_dict['add_url'] = self._get_admin_add_url(model, context)
@@ -287,7 +292,7 @@ class RecentActions(DashboardModule):
     Module that lists the recent actions for the current user.
     """
 
-    title = _('Recent Actions')
+    title = _('Recent actions')
     template = 'grappelli/dashboard/modules/recent_actions.html'
     limit = 10
     include_list = None
@@ -309,6 +314,7 @@ class RecentActions(DashboardModule):
         request = context['request']
 
         def get_qset(list):
+            from django.contrib.contenttypes.models import ContentType
             qset = None
             for contenttype in list:
                 if isinstance(contenttype, ContentType):
