@@ -10,7 +10,7 @@ To load the dashboard tags: ``{% load grp_dashboard_tags %}``.
 
 # DJANGO IMPORTS
 from django import template
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 
 # GRAPPELLI IMPORTS
 from grappelli.dashboard.utils import get_admin_site_name, get_index_dashboard
@@ -22,12 +22,12 @@ tag_func = register.inclusion_tag('grappelli/dashboard/dummy.html', takes_contex
 def grp_render_dashboard(context, location='index', dashboard=None):
     """
     Template tag that renders the dashboard, it takes two optional arguments:
-    
+
     ``location``
         The location of the dashboard, it can be 'index' (for the admin index
         dashboard) or 'app_index' (for the app index dashboard), the default
         value is 'index'.
-    
+
     ``dashboard``
         An instance of ``Dashboard``, if not given, the dashboard is retrieved
         with the ``get_index_dashboard`` or ``get_app_index_dashboard``
@@ -36,9 +36,9 @@ def grp_render_dashboard(context, location='index', dashboard=None):
 
     if dashboard is None:
         dashboard = get_index_dashboard(context)
-    
+
     dashboard.init_with_context(context)
-    
+
     context.update({
         'template': dashboard.template,
         'dashboard': dashboard,
@@ -54,7 +54,7 @@ def grp_render_dashboard_module(context, module, index=None, subindex=None):
     ``DashboardModule`` instance as first parameter and an integer ``index`` as
     second parameter, that is the index of the module in the dashboard.
     """
-    
+
     module.init_with_context(context)
     context.update({
         'template': module.template,
@@ -65,5 +65,3 @@ def grp_render_dashboard_module(context, module, index=None, subindex=None):
     })
     return context
 grp_render_dashboard_module = tag_func(grp_render_dashboard_module)
-
-
