@@ -26,7 +26,7 @@
             onAfterDeleted: function(form) {}       // Function called after a form has been deleted
         };
         options = $.extend(defaults, options);
-        
+
         return this.each(function() {
             var inline = $(this); // the current inline node
             var totalForms = inline.find("#id_" + options.prefix + "-TOTAL_FORMS");
@@ -47,7 +47,7 @@
         if (!formIndex) { return -1; }
         return parseInt(regex.exec(formIndex)[1], 10);
     };
-    
+
     updateFormIndex = function(elem, options, replace_regex, replace_with) {
         elem.find(':input,span,table,iframe,label,a,ul,p,img,div').each(function() {
             var node = $(this),
@@ -64,8 +64,24 @@
             if (node_class) { node.attr('class', node_class.replace(replace_regex, replace_with)); }
             if (node_onclick) { node.attr('onclick', node_onclick.replace(replace_regex, replace_with)); }
         });
+        // update prepopulate ids for function initPrepopulatedFields
+        elem.find('.prepopulated_field').each(function() {
+            var dependency_ids = $(this).data('dependency_ids') || [],
+                dependency_ids_updated = [];
+            $.each(dependency_ids, function(i, id) {
+                dependency_ids_updated.push(id.replace(replace_regex, replace_with));
+            });
+            $(this).data('dependency_ids', dependency_ids_updated);
+        });
     };
-    
+
+    var initPrepopulatedFields = function(elem, options) {
+        elem.find('.prepopulated_field').each(function() {
+            var dependency_ids = $(this).data('dependency_ids') || [];
+            $(this).prepopulate(dependency_ids, $(this).attr('maxlength'));
+        });
+    };
+
     initInlineForms = function(elem, options) {
         elem.find("div.grp-module").each(function() {
             var form = $(this);
@@ -86,7 +102,7 @@
             options.onAfterInit(form);
         });
     };
-    
+
     initAddButtons = function(elem, options) {
         var totalForms = elem.find("#id_" + options.prefix + "-TOTAL_FORMS");
         var maxForms = elem.find("#id_" + options.prefix + "-MAX_NUM_FORMS");
@@ -96,9 +112,9 @@
             hideAddButtons(elem, options);
         }
     };
-    
+
     addButtonHandler = function(elem, options) {
-        elem.bind("click", function() {
+        elem.on("click", function(e) {
             var inline = elem.parents(".grp-group"),
                 totalForms = inline.find("#id_" + options.prefix + "-TOTAL_FORMS"),
                 maxForms = inline.find("#id_" + options.prefix + "-MAX_NUM_FORMS"),
@@ -126,13 +142,17 @@
             if ((maxForms.val() !== 0) && (maxForms.val() !== "") && (maxForms.val() - totalForms.val()) <= 0) {
                 hideAddButtons(inline, options);
             }
+            // prepopulate fields
+            initPrepopulatedFields(form, options);
+            // select2: we need to use the django namespace here
+            django.jQuery(document).trigger('formset:added', [django.jQuery(form), options.prefix]);
             // callback
             options.onAfterAdded(form);
         });
     };
-    
+
     removeButtonHandler = function(elem, options) {
-        elem.bind("click", function() {
+        elem.on("click", function() {
             var inline = elem.parents(".grp-group"),
                 form = $(this).parents("." + options.formCssClass).first(),
                 totalForms = inline.find("#id_" + options.prefix + "-TOTAL_FORMS"),
@@ -161,9 +181,9 @@
             options.onAfterRemoved(inline);
         });
     };
-    
+
     deleteButtonHandler = function(elem, options) {
-        elem.bind("click", function() {
+        elem.on("click", function() {
             var deleteInput = $(this).prev(),
                 form = $(this).parents("." + options.formCssClass).first();
             // callback
@@ -181,19 +201,19 @@
             options.onAfterDeleted(form);
         });
     };
-    
+
     hideAddButtons = function(elem, options) {
         var addButtons = elem.find("a." + options.addCssClass);
         addButtons.hide().parents('.grp-add-item').hide();
         // last row with stacked/tabular
         addButtons.closest('.grp-module.grp-transparent').hide();
     };
-    
+
     showAddButtons = function(elem, options) {
         var addButtons = elem.find("a." + options.addCssClass);
         addButtons.show().parents('.grp-add-item').show();
         // last row with stacked/tabular
         addButtons.closest('.grp-module.grp-transparent').show();
     };
-    
+
 })(grp.jQuery);
