@@ -1,28 +1,18 @@
 # coding: utf-8
 
-# PYTHON IMPORTS
-import datetime
-
-# DJANGO IMPORTS
+from django.contrib.auth.models import User
 from django.test import TestCase
 from django.test.utils import override_settings
-from django.contrib.auth.models import User
-from django.core.urlresolvers import reverse
-from django.utils import translation
+from django.urls import reverse
+from django.utils import timezone, translation
 
-try:
-    import json
-except ImportError:
-    from django.utils import simplejson as json
-
-# GRAPPELLI IMPORTS
 from grappelli.tests.models import Category, Entry
 
 
 @override_settings(GRAPPELLI_AUTOCOMPLETE_LIMIT=10)
 @override_settings(GRAPPELLI_AUTOCOMPLETE_SEARCH_FIELDS={})
+@override_settings(ROOT_URLCONF="grappelli.tests.urls")
 class RelatedTests(TestCase):
-    urls = "grappelli.tests.urls"
 
     def setUp(self):
         """
@@ -42,10 +32,10 @@ class RelatedTests(TestCase):
 
         # add entries
         self.entry_superuser = Entry.objects.create(title="Entry Superuser",
-                                                    date=datetime.datetime.utcnow(),
+                                                    date=timezone.now(),
                                                     user=self.superuser_1)
         self.entry_editor = Entry.objects.create(title="Entry Editor",
-                                                 date=datetime.datetime.utcnow(),
+                                                 date=timezone.now(),
                                                  user=self.editor_1)
 
         # set to en to check error messages
@@ -70,53 +60,77 @@ class RelatedTests(TestCase):
         self.client.login(username="Superuser001", password="superuser001")
         response = self.client.get(reverse("grp_related_lookup"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": None, "label": ""}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": None, "label": ""}])
 
         # ok
         response = self.client.get("%s?object_id=1&app_label=%s&model_name=%s" % (reverse("grp_related_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "Category No 0 (1)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "Category No 0 (1)", "safe": False}])
+
+        # ok (to_field)
+        response = self.client.get("%s?object_id=1&to_field=id&app_label=%s&model_name=%s" % (reverse("grp_related_lookup"), "grappelli", "category"))
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "Category No 0 (1)", "safe": False}])
+
+        # ok (to_field)
+        response = self.client.get("%s?object_id=Category+No+0&to_field=name&app_label=%s&model_name=%s" % (reverse("grp_related_lookup"), "grappelli", "category"))
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "Category No 0", "label": "Category No 0 (1)", "safe": False}])
 
         # wrong object_id
         response = self.client.get("%s?object_id=10000&app_label=%s&model_name=%s" % (reverse("grp_related_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "10000", "label": "?"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "10000", "label": "?", "safe": False}])
+
+        # wrong object_id (to_field)
+        response = self.client.get("%s?object_id=xxx&to_field=name&app_label=%s&model_name=%s" % (reverse("grp_related_lookup"), "grappelli", "category"))
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "xxx", "label": "?", "safe": False}])
 
         # filtered queryset (single filter) fails
         response = self.client.get("%s?object_id=1&app_label=%s&model_name=%s&query_string=id__gte=99" % (reverse("grp_related_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "?"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "?", "safe": False}])
 
         # filtered queryset (single filter) works
         response = self.client.get("%s?object_id=100&app_label=%s&model_name=%s&query_string=id__gte=99" % (reverse("grp_related_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "100", "label": "Category No 99 (100)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "100", "label": "Category No 99 (100)", "safe": False}])
 
         # filtered queryset (multiple filters) fails
         response = self.client.get("%s?object_id=1&app_label=%s&model_name=%s&query_string=name__icontains=99:id__gte=99" % (reverse("grp_related_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "?"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "?", "safe": False}])
 
         # filtered queryset (multiple filters) works
         response = self.client.get("%s?object_id=100&app_label=%s&model_name=%s&query_string=name__icontains=99:id__gte=99" % (reverse("grp_related_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "100", "label": "Category No 99 (100)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "100", "label": "Category No 99 (100)", "safe": False}])
 
         # custom queryset (Superuser)
         response = self.client.get("%s?object_id=1&app_label=%s&model_name=%s" % (reverse("grp_related_lookup"), "grappelli", "entry"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "Entry Superuser"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "Entry Superuser", "safe": False}])
         response = self.client.get("%s?object_id=2&app_label=%s&model_name=%s" % (reverse("grp_related_lookup"), "grappelli", "entry"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "2", "label": "Entry Editor"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "2", "label": "Entry Editor", "safe": False}])
 
-        # cusotm queryset (Editor)
+        # custom queryset (Editor)
         # FIXME: this should fail, because the custom admin queryset
-        # limits the entry to the logged in superuser
+        # limits the entry to the logged in user (but we currently do not make use
+        # of custom admin querysets)
         self.client.login(username="Editor001", password="editor001")
         response = self.client.get("%s?object_id=1&app_label=%s&model_name=%s" % (reverse("grp_related_lookup"), "grappelli", "entry"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "Entry Superuser"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "Entry Superuser", "safe": False}])
+
+        # wrong app_label/model_name
+        response = self.client.get("%s?object_id=1&app_label=false&model_name=false" % (reverse("grp_related_lookup")))
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": None, "label": ""}])
+        response = self.client.get("%s?object_id=&app_label=false&model_name=false" % (reverse("grp_related_lookup")))
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": None, "label": ""}])
 
     def test_m2m_lookup(self):
         """
@@ -129,47 +143,47 @@ class RelatedTests(TestCase):
         self.client.login(username="Superuser001", password="superuser001")
         response = self.client.get(reverse("grp_related_lookup"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": None, "label": ""}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": None, "label": ""}])
 
         # ok (single)
         response = self.client.get("%s?object_id=1&app_label=%s&model_name=%s" % (reverse("grp_m2m_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "Category No 0 (1)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "Category No 0 (1)", "safe": False}])
 
         # wrong object_id (single)
         response = self.client.get("%s?object_id=10000&app_label=%s&model_name=%s" % (reverse("grp_m2m_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "10000", "label": "?"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "10000", "label": "?", "safe": False}])
 
         # ok (multiple)
         response = self.client.get("%s?object_id=1,2,3&app_label=%s&model_name=%s" % (reverse("grp_m2m_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "Category No 0 (1)"}, {"value": "2", "label": "Category No 1 (2)"}, {"value": "3", "label": "Category No 2 (3)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "Category No 0 (1)", "safe": False}, {"value": "2", "label": "Category No 1 (2)", "safe": False}, {"value": "3", "label": "Category No 2 (3)", "safe": False}])
 
         # wrong object_id (multiple)
         response = self.client.get("%s?object_id=1,10000,3&app_label=%s&model_name=%s" % (reverse("grp_m2m_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "Category No 0 (1)"}, {"value": "10000", "label": "?"}, {"value": "3", "label": "Category No 2 (3)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "Category No 0 (1)", "safe": False}, {"value": "10000", "label": "?", "safe": False}, {"value": "3", "label": "Category No 2 (3)", "safe": False}])
 
         # filtered queryset (single filter) fails
         response = self.client.get("%s?object_id=1,2,3&app_label=%s&model_name=%s&query_string=id__gte=99" % (reverse("grp_m2m_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "?"}, {"value": "2", "label": "?"}, {"value": "3", "label": "?"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "?", "safe": False}, {"value": "2", "label": "?", "safe": False}, {"value": "3", "label": "?", "safe": False}])
 
         # filtered queryset (single filter) works
         response = self.client.get("%s?object_id=1,2,3&app_label=%s&model_name=%s&query_string=id__lte=3" % (reverse("grp_m2m_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "Category No 0 (1)"}, {"value": "2", "label": "Category No 1 (2)"}, {"value": "3", "label": "Category No 2 (3)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "Category No 0 (1)", "safe": False}, {"value": "2", "label": "Category No 1 (2)", "safe": False}, {"value": "3", "label": "Category No 2 (3)", "safe": False}])
 
         # filtered queryset (multiple filters) fails
         response = self.client.get("%s?object_id=1,2,3&app_label=%s&model_name=%s&query_string=name__icontains=99:id__gte=99" % (reverse("grp_m2m_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "?"}, {"value": "2", "label": "?"}, {"value": "3", "label": "?"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "?", "safe": False}, {"value": "2", "label": "?", "safe": False}, {"value": "3", "label": "?", "safe": False}])
 
         # filtered queryset (multiple filters) works
         response = self.client.get("%s?object_id=1,2,3&app_label=%s&model_name=%s&query_string=name__icontains=Category:id__lte=3" % (reverse("grp_m2m_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": "1", "label": "Category No 0 (1)"}, {"value": "2", "label": "Category No 1 (2)"}, {"value": "3", "label": "Category No 2 (3)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": "1", "label": "Category No 0 (1)", "safe": False}, {"value": "2", "label": "Category No 1 (2)", "safe": False}, {"value": "3", "label": "Category No 2 (3)", "safe": False}])
 
     def test_autocomplete_lookup(self):
         """
@@ -182,24 +196,24 @@ class RelatedTests(TestCase):
         self.client.login(username="Superuser001", password="superuser001")
         response = self.client.get(reverse("grp_related_lookup"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": None, "label": ""}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": None, "label": ""}])
 
         # term not found
         response = self.client.get("%s?term=XXXXXXXXXX&app_label=%s&model_name=%s" % (reverse("grp_autocomplete_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": None, "label": "0 results"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": None, "label": "0 results"}])
 
         # ok (99 finds the id and the title, therefore 2 results)
         response = self.client.get("%s?term=Category No 99&app_label=%s&model_name=%s" % (reverse("grp_autocomplete_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": 99, "label": "Category No 98 (99)"}, {"value": 100, "label": "Category No 99 (100)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": 99, "label": "Category No 98 (99)"}, {"value": 100, "label": "Category No 99 (100)"}])
 
         # filtered queryset (single filter)
         response = self.client.get("%s?term=Category&app_label=%s&model_name=%s&query_string=id__gte=99" % (reverse("grp_autocomplete_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": 99, "label": "Category No 98 (99)"}, {"value": 100, "label": "Category No 99 (100)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": 99, "label": "Category No 98 (99)"}, {"value": 100, "label": "Category No 99 (100)"}])
 
         # filtered queryset (multiple filters)
         response = self.client.get("%s?term=Category&app_label=%s&model_name=%s&query_string=name__icontains=99:id__gte=99" % (reverse("grp_autocomplete_lookup"), "grappelli", "category"))
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, json.dumps([{"value": 100, "label": "Category No 99 (100)"}]))
+        self.assertJSONEqual(response.content.decode('utf-8'), [{"value": 100, "label": "Category No 99 (100)"}])
